import { Injectable } from '@angular/core';
import { SimulatedDeviceHelperService } from '../simulated-device-helper.service';
export var SimulatedAnalogLoggerService = (function () {
    function SimulatedAnalogLoggerService(_simulatedDeviceService) {
        // simulated MZ Scope has X channels while an MZ Logger has 8 channels
        this.offsets = [0, 0, 0, 0, 0, 0, 0, 0];
        this.gains = [1, 1, 1, 1, 1, 1, 1, 1];
        this.sampleFreqs = [10e6, 10e6, 10e6, 10e6, 10e6, 10e6, 10e6, 10e6];
        this.delays = [0, 0, 0, 0, 0, 0, 0, 0];
        this.maxSampleCounts = [1000, 1000, 1000, 1000, 1000, 1000, 1000, 1000];
        this.storageLocations = ["ram", "ram", "ram", "ram", "ram", "ram", "ram", "ram"];
        this.uris = ["", "", "", "", "", "", "", ""];
        this.states = ["stopped", "stopped", "stopped", "stopped", "stopped", "stopped", "stopped", "stopped"];
        this.startIndices = [0, 0, 0, 0, 0, 0, 0, 0]; // note(andrew): populate these values according to the # of channels the device has
        this.timeOfLastRead = [null, null, null, null, null, null, null, null];
        this.simulatedDeviceService = _simulatedDeviceService;
    }
    SimulatedAnalogLoggerService.prototype.getCurrentState = function (chan) {
        --chan; // note: channel is the channel number, not the index of the channel in our arrays. Subtracting one makes it the index.
        return {
            command: 'getCurrentState',
            statusCode: 0,
            state: this.states[chan],
            stopReason: "NORMAL",
            startIndex: this.startIndices[chan],
            actualCount: 0,
            maxSampleCount: -1,
            actualGain: this.gains[chan],
            actualVOffset: this.offsets[chan],
            actualSampleFreq: this.sampleFreqs[chan],
            actualStartDelay: this.delays[chan],
            overflow: "circular",
            storageLocation: "",
            uri: "",
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.setParameters = function (chan, commandObject) {
        --chan;
        var maxSampleCount = this.maxSampleCounts[chan] = commandObject.maxSampleCount;
        var actualGain = this.gains[chan] = commandObject.gain;
        var actualVOffset = this.offsets[chan] = commandObject.vOffset;
        var actualSampleFreq = this.sampleFreqs[chan] = commandObject.sampleFreq;
        var actualStartDelay = this.delays[chan] = commandObject.startDelay;
        var storageLocation = this.storageLocations[chan] = commandObject.storageLocation;
        var uri = this.uris[chan] = commandObject.uri;
        return {
            command: "setParameters",
            statusCode: 0,
            maxSampleCount: maxSampleCount,
            actualGain: actualGain,
            actualVOffset: actualVOffset,
            actualSampleFreq: actualSampleFreq,
            actualStartDelay: actualStartDelay,
            storageLocation: storageLocation,
            uri: uri,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.run = function (chan) {
        --chan;
        this.timeOfLastRead[chan] = Date.now();
        this.states[chan] = "running";
        this.startIndices[chan] = 0;
        return {
            command: "run",
            statusCode: 0,
            wait: -1
        };
    };
    SimulatedAnalogLoggerService.prototype.stop = function (chan) {
        --chan;
        this.states[chan] = "stopped";
        return {
            command: "stop",
            statusCode: 0,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.drawSine = function (awgSettings, chan) {
        var now = Date.now();
        var T = now - this.timeOfLastRead[chan];
        this.timeOfLastRead[chan] = now;
        // grab sample rate from the AWG
        var sigFreq = awgSettings.signalFreq;
        var vpp = awgSettings.vpp;
        var sampleRate = this.sampleFreqs[chan] || 6.25e9;
        var numSamples = (sampleRate / 1e6) * (T / 1000);
        var vOffset = awgSettings.vOffset;
        var t0 = this.startIndices[chan];
        var dt = 1e6 / sampleRate;
        var phase = (chan * 90) * (Math.PI / 180); // in radians
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            y[i] = (vpp / 2) * Math.sin((2 * Math.PI * (sigFreq / 1000)) * dt * (i + t0) + phase) + vOffset;
        }
        console.log("Y:", y);
        var typedArray = new Int16Array(y);
        return {
            command: "read",
            statusCode: 0,
            binaryOffset: 0,
            binaryLength: 2 * typedArray.length,
            actualCount: typedArray.length,
            startIndex: this.startIndices[chan],
            maxSampleCount: this.maxSampleCounts[chan],
            actualGain: this.gains[chan],
            actualVOffset: vOffset,
            actualSampleFreq: 1e6 / dt,
            actualStartDelay: 0,
            storageLocation: "ram",
            uri: "",
            y: typedArray,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.drawSquare = function (awgSettings, chan) {
        var now = Date.now();
        var T = now - this.timeOfLastRead[chan];
        this.timeOfLastRead[chan] = now;
        //Set default values 
        var sigFreq = awgSettings.signalFreq;
        var vpp = awgSettings.vpp;
        var sampleRate = this.sampleFreqs[chan] || 6.25e9;
        var numSamples = (sampleRate / 1e6) * (T / 1000);
        var vOffset = awgSettings.vOffset;
        var t0 = this.startIndices[chan];
        var dutyCycle = 50;
        var dt = 1e6 / sampleRate;
        var period = 1 / (sigFreq / 1000);
        var phase = chan * (period / 4);
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            if ((dt * (i + t0) + phase) % period < period * (dutyCycle / 100)) {
                y[i] = (vOffset + vpp / 2);
            }
            else {
                y[i] = (vOffset - vpp / 2);
            }
        }
        console.log("Y:", y);
        var typedArray = new Int16Array(y);
        //length is 2x the array length because 2 bytes per entry
        return {
            command: "read",
            statusCode: 0,
            binaryOffset: 0,
            binaryLength: 2 * typedArray.length,
            actualCount: typedArray.length,
            startIndex: this.startIndices[chan],
            maxSampleCount: this.maxSampleCounts[chan],
            actualGain: this.gains[chan],
            actualVOffset: vOffset,
            actualSampleFreq: 1e6 / dt,
            actualStartDelay: 0,
            storageLocation: "ram",
            uri: "",
            y: typedArray,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.drawTriangle = function (awgSettings, chan) {
        var now = Date.now();
        var T = now - this.timeOfLastRead[chan];
        this.timeOfLastRead[chan] = now;
        var sigFreq = awgSettings.signalFreq;
        var vpp = awgSettings.vpp;
        var sampleRate = this.sampleFreqs[chan] || 6.25e9;
        var numSamples = (sampleRate / 1e6) * (T / 1000);
        var vOffset = awgSettings.vOffset;
        var t0 = this.startIndices[chan];
        var dt = 1e6 / sampleRate;
        var period = 1 / (sigFreq / 1000);
        var phase = chan * (period / 4);
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            y[i] = ((4 * (vpp / 2)) / period) * (Math.abs(((dt * (i + t0) + phase + 3 * period / 4) % period) - period / 2) - period / 4) + vOffset;
        }
        console.log("Y:", y);
        var typedArray = new Int16Array(y);
        return {
            command: "read",
            statusCode: 0,
            binaryOffset: 0,
            binaryLength: 2 * typedArray.length,
            actualCount: typedArray.length,
            startIndex: this.startIndices[chan],
            maxSampleCount: this.maxSampleCounts[chan],
            actualGain: this.gains[chan],
            actualVOffset: vOffset,
            actualSampleFreq: 1e6 / dt,
            actualStartDelay: 0,
            storageLocation: "ram",
            uri: "",
            y: typedArray,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.drawSawtooth = function (awgSettings, chan) {
        var now = Date.now();
        var T = now - this.timeOfLastRead[chan];
        this.timeOfLastRead[chan] = now;
        var sigFreq = awgSettings.signalFreq; //in mHz
        var vpp = awgSettings.vpp; //mV
        var sampleRate = this.sampleFreqs[chan] || 6.25e9;
        var numSamples = (sampleRate / 1e6) * (T / 1000);
        var vOffset = awgSettings.vOffset; //in mV
        var t0 = this.startIndices[chan];
        //Calculate dt - time between data points
        var dt = 1e6 / sampleRate;
        var period = 1 / (sigFreq / 1000);
        var phase = chan * (period / 4);
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            y[i] = (vpp / period) * ((dt * (i + t0) + phase) % period) + vOffset;
        }
        console.log("Y:", y);
        var typedArray = new Int16Array(y);
        return {
            command: "read",
            statusCode: 0,
            binaryOffset: 0,
            binaryLength: 2 * typedArray.length,
            actualCount: typedArray.length,
            startIndex: this.startIndices[chan],
            maxSampleCount: this.maxSampleCounts[chan],
            actualGain: this.gains[chan],
            actualVOffset: vOffset,
            actualSampleFreq: 1e6 / dt,
            actualStartDelay: 0,
            storageLocation: "ram",
            uri: "",
            y: typedArray,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.drawDc = function (awgSettings, chan) {
        var now = Date.now();
        var T = now - this.timeOfLastRead[chan];
        this.timeOfLastRead[chan] = now;
        var sampleRate = this.sampleFreqs[chan] || 6.25e9;
        var numSamples = (sampleRate / 1e6) * (T / 1000);
        var vOffset = awgSettings.vOffset;
        var dt = 1e6 / sampleRate;
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            y[i] = vOffset;
        }
        console.log("Y:", y);
        var typedArray = new Int16Array(y);
        return {
            command: "read",
            statusCode: 0,
            binaryOffset: 0,
            binaryLength: 2 * typedArray.length,
            actualCount: typedArray.length,
            startIndex: this.startIndices[chan],
            maxSampleCount: this.maxSampleCounts[chan],
            actualGain: this.gains[chan],
            actualVOffset: vOffset,
            actualSampleFreq: 1e6 / dt,
            actualStartDelay: 0,
            storageLocation: "ram",
            uri: "",
            y: typedArray,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.drawDefault = function (chan) {
        var now = Date.now();
        var T = now - this.timeOfLastRead[chan];
        this.timeOfLastRead[chan] = now;
        var sampleRate = this.sampleFreqs[chan] || 6.25e9;
        var numSamples = (sampleRate / 1e6) * (T / 1000);
        var vOffset = 0;
        var dt = 1e6 / sampleRate;
        var y = [];
        for (var j = 0; j < numSamples; j++) {
            y[j] = 0;
        }
        console.log("Y:", y);
        var typedArray = new Int16Array(y);
        return {
            command: "read",
            statusCode: 0,
            binaryOffset: 0,
            binaryLength: 2 * typedArray.length,
            actualCount: typedArray.length,
            startIndex: this.startIndices[chan],
            maxSampleCount: this.maxSampleCounts[chan],
            actualGain: this.gains[chan],
            actualVOffset: vOffset,
            actualSampleFreq: 1e6 / dt,
            actualStartDelay: 0,
            storageLocation: "ram",
            uri: "",
            y: typedArray,
            wait: 0
        };
    };
    SimulatedAnalogLoggerService.prototype.read = function (chan) {
        --chan;
        var awgSettings = this.simulatedDeviceService.getAwgSettings(1);
        console.log("AWG Settings:", awgSettings);
        var responseObj;
        switch (awgSettings.signalType) {
            case 'sine':
                responseObj = this.drawSine(awgSettings, chan);
                break;
            case 'triangle':
                responseObj = this.drawTriangle(awgSettings, chan);
                console.log(responseObj);
                break;
            case 'sawtooth':
                responseObj = this.drawSawtooth(awgSettings, chan);
                break;
            case 'square':
                responseObj = this.drawSquare(awgSettings, chan);
                console.log(responseObj);
                break;
            case 'dc':
                responseObj = this.drawDc(awgSettings, chan);
                break;
            default:
                responseObj = this.drawDefault(chan);
                break;
        }
        this.startIndices[chan] += responseObj.actualCount;
        return responseObj;
    };
    SimulatedAnalogLoggerService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SimulatedAnalogLoggerService.ctorParameters = [
        { type: SimulatedDeviceHelperService, },
    ];
    return SimulatedAnalogLoggerService;
}());
//# sourceMappingURL=simulated-analog-logger.service.js.map