import { Injectable } from '@angular/core';
import { SimulatedDeviceHelperService } from '../simulated-device-helper.service';
export var SimulatedDaqLoggerService = (function () {
    function SimulatedDaqLoggerService(_simulatedDeviceService) {
        this.sampleFreq = 10e8;
        this.delay = 0;
        this.maxSampleCount = 1000;
        this.averages = [1, 1, 1, 1, 1, 1, 1, 1];
        this.storageLocation = "ram";
        this.uri = "";
        this.state = "stopped";
        this.startIndex = 0;
        this.timeOfLastRead = null;
        this.selectedChannels = [true, false, false, false, false, false, false, false];
        this.simulatedDeviceService = _simulatedDeviceService;
    }
    SimulatedDaqLoggerService.prototype.getCurrentState = function () {
        var _this = this;
        var channels = [];
        this.selectedChannels.forEach(function (chan, index) {
            if (chan) {
                channels.push((_a = {},
                    _a[(index + 1).toString()] = {
                        average: _this.averages[index]
                    },
                    _a
                ));
            }
            var _a;
        });
        return {
            command: 'getCurrentState',
            statusCode: 0,
            state: this.state,
            stopReason: "NORMAL",
            startIndex: this.startIndex,
            actualCount: 0,
            maxSampleCount: -1,
            actualSampleFreq: this.sampleFreq,
            actualStartDelay: this.delay,
            storageLocation: this.storageLocation,
            uri: this.uri,
            channels: channels,
            wait: 0
        };
    };
    SimulatedDaqLoggerService.prototype.setParameters = function (commandObject) {
        var _this = this;
        var maxSampleCount = this.maxSampleCount = commandObject.maxSampleCount;
        var actualSampleFreq = this.sampleFreq = commandObject.sampleFreq;
        var actualStartDelay = this.delay = commandObject.startDelay;
        var storageLocation = this.storageLocation = commandObject.storageLocation;
        var uri = this.uri = commandObject.uri;
        // select channels and set params
        this.selectedChannels = this.selectedChannels.map(function () { return false; });
        commandObject.channels.forEach(function (chan) {
            var chanNum = parseInt(Object.keys(chan)[0]);
            _this.selectedChannels[chanNum - 1] = true;
            _this.averages[chan - 1] = chan[chanNum].average;
        });
        return {
            command: "setParameters",
            statusCode: 0,
            maxSampleCount: maxSampleCount,
            actualSampleFreq: actualSampleFreq,
            actualStartDelay: actualStartDelay,
            storageLocation: storageLocation,
            uri: uri,
            channels: commandObject.channels,
            wait: 0
        };
    };
    SimulatedDaqLoggerService.prototype.run = function () {
        this.timeOfLastRead = Date.now();
        this.state = "running";
        this.startIndex = 0;
        return {
            command: "run",
            statusCode: 0,
            wait: 0
        };
    };
    SimulatedDaqLoggerService.prototype.stop = function () {
        this.state = "stopped";
        return {
            command: "stop",
            statusCode: 0,
            wait: 0
        };
    };
    SimulatedDaqLoggerService.prototype.drawSine = function (awgSettings, channels, numSamples, dt) {
        var sigFreq = awgSettings.signalFreq;
        var vpp = awgSettings.vpp;
        var vOffset = awgSettings.vOffset;
        var t0 = this.startIndex;
        var phases = [];
        channels.forEach(function (chan) {
            phases.push((chan * 90) * (Math.PI / 180)); // in radians
        });
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            for (var j = 0; j < channels.length; j++) {
                y.push((vpp / 2) * Math.sin((2 * Math.PI * (sigFreq / 1000)) * dt * (i + t0) + phases[j]) + vOffset);
            }
        }
        return new Int16Array(y);
    };
    SimulatedDaqLoggerService.prototype.drawSquare = function (awgSettings, channels, numSamples, dt) {
        var sigFreq = awgSettings.signalFreq;
        var vpp = awgSettings.vpp;
        var vOffset = awgSettings.vOffset;
        var t0 = this.startIndex;
        var dutyCycle = 50;
        var period = 1 / (sigFreq / 1000);
        var phases = [];
        channels.forEach(function (chan) {
            phases.push(chan * (period / 4));
        });
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            for (var j = 0; j < channels.length; j++) {
                if ((dt * (i + t0) + phases[j]) % period < period * (dutyCycle / 100)) {
                    y.push(vOffset + vpp / 2);
                }
                else {
                    y.push(vOffset - vpp / 2);
                }
            }
        }
        return new Int16Array(y);
    };
    SimulatedDaqLoggerService.prototype.drawTriangle = function (awgSettings, channels, numSamples, dt) {
        var sigFreq = awgSettings.signalFreq;
        var vpp = awgSettings.vpp;
        var vOffset = awgSettings.vOffset;
        var t0 = this.startIndex;
        var period = 1 / (sigFreq / 1000);
        var phases = [];
        channels.forEach(function (chan) {
            phases.push(chan * (period / 4));
        });
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            for (var j = 0; j < channels.length; j++) {
                y.push(((4 * (vpp / 2)) / period) * (Math.abs(((dt * (i + t0) + phases[j] + 3 * period / 4) % period) - period / 2) - period / 4) + vOffset);
            }
        }
        return new Int16Array(y);
    };
    SimulatedDaqLoggerService.prototype.drawSawtooth = function (awgSettings, channels, numSamples, dt) {
        var sigFreq = awgSettings.signalFreq; //in mHz
        var vpp = awgSettings.vpp; //mV
        var vOffset = awgSettings.vOffset; //in mV
        var t0 = this.startIndex;
        var period = 1 / (sigFreq / 1000);
        var phases = [];
        channels.forEach(function (chan) {
            phases.push(chan * (period / 4));
        });
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            for (var j = 0; j < channels.length; j++) {
                y.push((vpp / period) * ((dt * (i + t0) + phases[j]) % period) + vOffset);
            }
        }
        return new Int16Array(y);
    };
    SimulatedDaqLoggerService.prototype.drawDc = function (vOffset, channels, numSamples) {
        var y = [];
        for (var i = 0; i < numSamples; i++) {
            for (var j = 0; j < channels.length; j++) {
                y.push(vOffset);
            }
        }
        return new Int16Array(y);
    };
    SimulatedDaqLoggerService.prototype.drawDefault = function (channels, numSamples) {
        var y = [];
        for (var j = 0; j < numSamples; j++) {
            for (var i = 0; i < channels.length; i++) {
                y.push(0);
            }
        }
        return new Int16Array(y);
    };
    SimulatedDaqLoggerService.prototype.read = function (commandObject) {
        var awgSettings = this.simulatedDeviceService.getAwgSettings(1);
        var channels = commandObject.channels;
        var now = Date.now();
        var T = now - this.timeOfLastRead;
        var sampleRate = this.sampleFreq || 6.25e9;
        var numSamples = Math.round((sampleRate / 1e6) * (T / 1000));
        if (numSamples > 0) {
            this.timeOfLastRead = now;
        }
        var dt = 1e6 / sampleRate;
        var responseObj = {
            command: "read",
            statusCode: 0,
            channels: channels,
            binaryOffset: 0,
            binaryLength: null,
            actualCount: null,
            startIndex: this.startIndex,
            maxSampleCount: this.maxSampleCount,
            actualGain: 1,
            actualVOffset: awgSettings.vOffset,
            actualSampleFreq: 1e6 / dt,
            actualStartDelay: 0,
            overflow: "circular",
            y: null,
            wait: 0
        };
        var typedArray;
        if (!this.simulatedDeviceService.getTriggerArmed()) {
            typedArray = this.drawDefault(channels, numSamples);
            responseObj.actualVOffset = 0;
        }
        else {
            switch (awgSettings.signalType) {
                case 'sine':
                    typedArray = this.drawSine(awgSettings, channels, numSamples, dt);
                    break;
                case 'triangle':
                    typedArray = this.drawTriangle(awgSettings, channels, numSamples, dt);
                    break;
                case 'sawtooth':
                    typedArray = this.drawSawtooth(awgSettings, channels, numSamples, dt);
                    break;
                case 'square':
                    typedArray = this.drawSquare(awgSettings, channels, numSamples, dt);
                    break;
                case 'dc':
                    typedArray = this.drawDc(awgSettings.vOffset, channels, numSamples);
                    break;
                default:
                    typedArray = this.drawDefault(channels, numSamples);
                    responseObj.actualVOffset = 0;
                    break;
            }
        }
        responseObj.binaryLength = 2 * typedArray.length;
        responseObj.actualCount = typedArray.length / commandObject.channels.length;
        responseObj.y = typedArray;
        this.startIndex += responseObj.actualCount;
        return responseObj;
    };
    SimulatedDaqLoggerService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SimulatedDaqLoggerService.ctorParameters = [
        { type: SimulatedDeviceHelperService, },
    ];
    return SimulatedDaqLoggerService;
}());
//# sourceMappingURL=simulated-daq-logger.service.js.map