import { Injectable } from '@angular/core';
import { Observable } from 'rxjs/Observable';
import 'rxjs/Rx';
import { SimulatedAwgService } from './instruments/simulated-awg.service';
import { SimulatedDcService } from './instruments/simulated-dc.service';
import { SimulatedDaqLoggerService } from './instruments/simulated-daq-logger.service';
import { SimulatedDeviceHelperService } from './simulated-device-helper.service';
import { CommandUtilityService } from '../utilities/command-utility.service';
export var SimulatedOpenLoggerService = (function () {
    function SimulatedOpenLoggerService(enumeration) {
        this.descriptor = enumeration;
        this.commandUtilityService = new CommandUtilityService();
        this.simDevService = new SimulatedDeviceHelperService();
        this.simDevService.setEnumeration(this.descriptor);
        this.awg = new SimulatedAwgService(this.simDevService);
        this.dc = new SimulatedDcService(this.simDevService);
        this.logger = new SimulatedDaqLoggerService(this.simDevService);
    }
    SimulatedOpenLoggerService.prototype.send = function (command) {
        var _this = this;
        return Observable.create(function (observer) {
            observer.next(_this.parseCommand(JSON.parse(command)));
            observer.complete();
        });
    };
    SimulatedOpenLoggerService.prototype.parseCommand = function (event) {
        var _this = this;
        var responseObject = {};
        var sumStatusCode = 0;
        var binaryDataFlag = 0;
        var _loop_1 = function(instrument) {
            //create property on response object
            responseObject[instrument] = {};
            if (event[instrument][0] !== undefined && event[instrument][0].command !== undefined) {
                if (instrument === 'device') {
                    responseObject[instrument] = [];
                    var activeIndex = responseObject[instrument].push(this_1.processCommands(instrument, event[instrument][0], [])) - 1;
                    sumStatusCode += responseObject[instrument][activeIndex].statusCode;
                }
                else {
                    responseObject[instrument] = this_1.processCommands(instrument, event[instrument][0], []);
                    sumStatusCode += responseObject[instrument].statusCode;
                }
            }
            else if (typeof event[instrument] === 'object') {
                if (instrument === 'log') {
                    var daqObj = event[instrument].daq;
                    responseObject[instrument]['daq'] = this_1.processCommands(instrument, daqObj, []);
                    sumStatusCode += responseObject[instrument].daq.statusCode;
                    if (daqObj.command === "read") {
                        binaryDataFlag = 1;
                    }
                    return "continue"; // skip doing loop2 because it doesn't pertain to the logger 
                }
            }
            var _loop_2 = function(channel) {
                if (event[instrument][channel][0] !== undefined) {
                    //create property on response object 
                    responseObject[instrument][channel] = [];
                    event[instrument][channel].forEach(function (element, index, array) {
                        responseObject[instrument][channel].push(_this.processCommands(instrument, event[instrument][channel][index], [channel])) - 1;
                        if (element.command === 'read' && instrument !== 'gpio') {
                            binaryDataFlag = 1;
                        }
                    });
                }
            };
            for (var channel in event[instrument]) {
                _loop_2(channel);
            }
        };
        var this_1 = this;
        for (var instrument in event) {
            _loop_1(instrument);
        }
        if (binaryDataFlag) {
            return this.processBinaryDataAndSend(responseObject);
        }
        else {
            var response = JSON.stringify(responseObject);
            var buf = new ArrayBuffer(response.length);
            var bufView = new Uint8Array(buf);
            for (var i = 0; i < response.length; i++) {
                bufView[i] = response.charCodeAt(i);
            }
            return bufView.buffer;
        }
    };
    SimulatedOpenLoggerService.prototype.processCommands = function (instrument, commandObject, params) {
        var command = instrument + commandObject.command;
        switch (command) {
            //---------- Device ----------
            case 'deviceenumerate':
                return this.descriptor;
            case 'devicestorageGetLocations':
                return {
                    command: 'storageGetLocations',
                    statusCode: 0,
                    storageLocations: [],
                    wait: 0
                };
            //---------- AWG ----------            
            case 'awgsetArbitraryWaveform':
                return this.awg.setArbitraryWaveform(params[0]);
            case 'awgsetRegularWaveform':
                return this.awg.setRegularWaveform(params[0], commandObject);
            case 'awggetCurrentState':
                return this.awg.getCurrentState(params[0]);
            case 'awgrun':
                return this.awg.run(params[0]);
            case 'awgstop':
                return this.awg.stop(params[0]);
            //---------- DC ----------        
            case 'dcsetVoltage':
                return this.dc.setVoltage(params[0], commandObject.voltage);
            case 'dcgetVoltage':
                return this.dc.getVoltage(params[0]);
            //---------- GPIO ----------        
            case 'gpiowrite':
                return this.gpio.write(params[0], commandObject.value);
            case 'gpioread':
                return this.gpio.read(params[0]);
            case 'gpiosetParameters':
                return this.gpio.setParameters(params[0], commandObject.direction);
            //-------- TRIGGER --------
            case 'triggersetParameters':
                return this.trigger.setParameters(params[0], commandObject.source, commandObject.targets);
            case 'triggersingle':
                return this.trigger.single();
            case 'triggerforceTrigger':
                return this.trigger.forceTrigger();
            case 'triggerstop':
                return this.trigger.stop();
            //---------- OSC ----------            
            case 'oscsetParameters':
                return this.osc.setParameters(params[0], commandObject);
            case 'oscread':
                return this.osc.read(params[0]);
            //---------- LA ----------            
            case 'lasetParameters':
                return this.la.setParameters(params[0], commandObject);
            case 'laread':
                return this.la.read(params[0]);
            //---------- LOGGER ----------  
            case 'loggetCurrentState':
                return this.logger.getCurrentState();
            case 'logstop':
                return this.logger.stop();
            case 'logsetParameters':
                return this.logger.setParameters(commandObject);
            case 'logrun':
                return this.logger.run();
            case 'logread':
                return this.logger.read(commandObject);
            case 'filelistdir':
                return [{
                        command: 'listdir',
                        files: [],
                        path: '/',
                        statusCode: 0,
                        type: 'flash',
                        wait: 0
                    }];
            case 'filegetCurrentState':
                return {
                    command: "getCurrentState",
                    statusCode: 0,
                    wait: 0
                };
            default:
                return {
                    statusCode: 1,
                    errorMessage: command + " not a recognized command"
                };
        }
    };
    SimulatedOpenLoggerService.prototype.processBinaryDataAndSend = function (commandObject) {
        var binaryDataContainer = {};
        var binaryOffset = 0;
        // note(andrew): if the container is empty then that means the commandObject instrument
        // is the logger, because the trigger doesn't get set when logging, and doesn't
        // ever get 'log' as a target
        if (Object.keys(binaryDataContainer).length === 0 && commandObject["log"] !== undefined) {
            binaryDataContainer["log"] = {};
            for (var channelType in commandObject["log"]) {
                binaryDataContainer.log[channelType] = commandObject.log[channelType].y;
                commandObject.log[channelType].binaryOffset = binaryOffset;
                binaryOffset += commandObject.log[channelType].binaryLength;
                delete commandObject.log[channelType].y;
            }
        }
        var buf = new ArrayBuffer(binaryOffset);
        var bufView = new Uint8Array(buf);
        var binaryInjectorIndex = 0;
        var prevLength = 0;
        for (var instrument in binaryDataContainer) {
            if (instrument === "log") {
                for (var channelType in binaryDataContainer.log) {
                    var unsignedConversion = new Uint8Array(binaryDataContainer.log[channelType].buffer);
                    binaryInjectorIndex += prevLength + unsignedConversion.length;
                    for (var i = prevLength, j = 0; i < binaryInjectorIndex; i = i + 2, j = j + 2) {
                        bufView[i] = unsignedConversion[j];
                        bufView[i + 1] = unsignedConversion[j + 1];
                    }
                    prevLength = unsignedConversion.length;
                }
            }
            else {
                for (var channel in binaryDataContainer[instrument]) {
                    var unsignedConversion = new Uint8Array(binaryDataContainer[instrument][channel].buffer);
                    binaryInjectorIndex += prevLength + unsignedConversion.length;
                    for (var i = prevLength, j = 0; i < binaryInjectorIndex; i = i + 2, j = j + 2) {
                        bufView[i] = unsignedConversion[j];
                        bufView[i + 1] = unsignedConversion[j + 1];
                    }
                    prevLength = unsignedConversion.length;
                    if (instrument === 'la') {
                        break;
                    }
                }
            }
        }
        return this.commandUtilityService.createChunkedArrayBuffer(commandObject, bufView.buffer).buffer;
    };
    SimulatedOpenLoggerService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    SimulatedOpenLoggerService.ctorParameters = [
        null,
    ];
    return SimulatedOpenLoggerService;
}());
//# sourceMappingURL=simulated-openlogger.service.js.map